/*
 **************************************************************************
 *                                                                        *
 *          General Purpose Hash Function Algorithms Library              *
 *                                                                        *
 * Author: Arash Partow - 2002                                            *
 * Class: Bloom Filter                                                    *
 * URL: http://www.partow.net                                             *
 * URL: http://www.partow.net/programming/hashfunctions/index.html        *
 *                                                                        *
 * Copyright notice:                                                      *
 * Free use of the General Purpose Hash Function Algorithms Library is    *
 * permitted under the guidelines and in accordance with the most current *
 * version of the Common Public License.                                  *
 * http://www.opensource.org/licenses/cpl.php                             *
 *                                                                        *
 **************************************************************************
*/

/*
	Slight modifications made my Dan Wendlandt to use for the kernel
	and userspace implementation of a Fastpass router.  6-02-2006
*/

#ifndef INCLUDE_BLOOM_FILTER_H
#define INCLUDE_BLOOM_FILTER_H


#define NUM_HASH_FUNCS	8

#include "GeneralHashFunctions.h"


#ifdef CLICK_LINUXMODULE
#include <linux/string.h>
#else
#include <string.h>
#endif

const unsigned int  char_size   = 0x08;    // 8 bits in 1 char(unsigned)
const unsigned char bit_mask[8] = {
                                    0x01,  //00000001
                                    0x02,  //00000010
                                    0x04,  //00000100
                                    0x08,  //00001000
                                    0x10,  //00010000
                                    0x20,  //00100000
                                    0x40,  //01000000
                                    0x80   //10000000
                                 };

 class bloom_filter
 {

    public:

      bloom_filter(unsigned int tbl_size)
      {
         table_size = tbl_size;
         hash_table = new unsigned char[table_size];
         register_default_hash_functions();
       //  for (unsigned int i = 0; i < table_size; i++)
        // {
        //    hash_table[i] = (unsigned char)0;
        // }
	clear();
      }

      ~bloom_filter()
      {
         delete[] hash_table;
      }

	// clear all bloom filter entries
	void clear() {
		memset(hash_table, 0x0, table_size);
	}

      void register_default_hash_functions()
      {
	funcs[0] = RSHash;
	funcs[1] = JSHash;
	funcs[2] = PJWHash;
	funcs[3] = BKDRHash;
	funcs[4] = SDBMHash;
	funcs[5] = DJBHash;
	funcs[6] = DEKHash;
	funcs[7] = APHash;
      }

      void insert(const char * key, int len)
      {
         for(unsigned int i = 0; i < NUM_HASH_FUNCS; i++)
         {
            unsigned int bit_index = funcs[i](key,len) 
					% (table_size * char_size);
            hash_table[bit_index / char_size] |= 
			bit_mask[bit_index % char_size];
         }
      }

      bool contains(const char *key, int len)
      {
         for(unsigned int i = 0; i < NUM_HASH_FUNCS; i++)
         {
            unsigned int bit_index = funcs[i](key,len) 
					% (table_size * char_size);
            unsigned int bit       = bit_index % char_size;
            if ((hash_table[bit_index / char_size] & bit_mask[bit]) 
			!= bit_mask[bit])
            {
               return false;
            }
         }
         return true;
      }

    private:
      HashFunction funcs[NUM_HASH_FUNCS];
      unsigned char*               hash_table;
      unsigned int                 table_size;

 };

#endif
