#include <click/config.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/router.hh>
#include <click/standard/scheduleinfo.hh>
#include <click/glue.hh>
#include <clicknet/ip.h>

#include "fastpass_common.hh"
#include "cap_granter.h"
//#include "time_utils.h"


/*
  Initialize state for the capability granter
*/
void granter_init(secflow_granter* granter){	
	
	granter -> has_cap_reply = 0;
	memset(&granter -> latest_cap_reply, 0, sizeof(secflow_capability));

	granter -> app_grant_class = MAXNUMCLASS - 1;
	granter -> no_timer = 1;
	
}

/*
  When the granter receives a packet, it checks to see if the contained capability is 
  valid, and if so it saves this value to echo to the requester.  
*/
void granter_process_hdr(secflow_granter* granter, hdr_secflow *hdr){
        char cap_is_newer = 0, class_is_allowed = 0;

	assert(granter != NULL);
	assert(hdr != NULL);
	
	//	cap_is_newer = compare_capability(&granter->latest_cap_reply, &(hdr->cap_next)) > 0;
	class_is_allowed = granter -> app_grant_class >= getprio(&hdr->cap_next);
	//	printk("GRANTER: process_hdr  is_request = %d request_failed = %d priority = %d app_grant_class = %d \n",
	//   is_request(hdr), request_failed(hdr), getprio(&hdr->cap_next), granter->app_grant_class);
	if( is_request(hdr) && !request_failed(hdr) &&
	    getprio(&hdr->cap_next) > 0 && class_is_allowed /* && cap_is_newer */ ){
	  // received new and valid capability


		if(granter->has_cap_reply == 0){
                        struct timeval now;
                        click_gettimeofday(&now);
                        //click_chatter("RECEIVED INITIAL REQUEST, recv: now.tv_sec = %u, now.tv_usec = %u\n", 
			  //     (u32)now.tv_sec, (u32)now.tv_usec);                                     
                }
		
		// store the capability just received in this packet, to return the sender
		memcpy(&granter->latest_cap_reply, &(hdr->cap_next), sizeof(secflow_capability));
	
		setcurrouter(&granter->latest_cap_reply, 0);
		granter -> has_cap_reply = 1;
     
//		printf("granter: got new capability to ECHO, id = %d\n", hdr->cap_next.id);
		
		//if no app data in SECFLOW_GRANTER_ACK_TIMEOUT, actively initiates an ACK.
		if(granter -> no_timer){
			granter -> no_timer = 0;
		}
	      
	}
	
}


/*
  As we are about to return a packet to the capability requester. 
  We copy the latest capability received into the capability reply field
  And just set the priority.  
*/
void granter_initialize_hdr(secflow_granter* granter, hdr_secflow *hdr){
	
	assert(granter != NULL);
	assert(hdr != NULL);

	// always copy the capability into the packet?  
	// dw: shouldn't we decide if we like these people or not?  

	memcpy(&hdr -> cap_reply, &granter -> latest_cap_reply, sizeof(secflow_capability));
	setcurrouter(&hdr -> cap_reply, 0);

	if(granter -> has_cap_reply && 
	   getprio(&granter->latest_cap_reply) <= granter -> app_grant_class){
		set_cap_reply_bit(hdr);

		//hack: give back highest class	granter is willing to give
		setprio(&hdr -> cap_reply, granter -> app_grant_class);
//		printf("granter: attaching capability reply, id = %d \n", hdr -> cap_reply.id);
	}
	
	//clear timeout
	granter -> no_timer  = 1;
	
}

/*
  cancel timer
*/
void granter_cleanup(secflow_granter* granter){

}

CLICK_ENDDECLS

ELEMENT_PROVIDES(bcap_granter)
