
#include "fastpass_common.hh"
#include "cap_requester.h"
//#include "cap_timer.h"
//#include "time_utils.h"

/*
runting:
application specifies app_req_class, the maximum class the application wants.
upon receiving a cap_reply for the capability I am currently requesting, the requester
increments the requesting class, unless it's already requesting app_req_class.
upon timeout, the requester decrement the requesting class
*/



/* 
   initialize requester data-structure
   begin by requesting the maximum class
*/ 
void requester_init(secflow_requester* requester){
	assert(requester != NULL);

	memset(&requester -> latest_cap, 0, sizeof(secflow_capability));
	requester -> app_req_class = MAXNUMCLASS - 1;
	requester -> requesting_class = MAXNUMCLASS - 1;
	
	requester -> first_req_4_cur_class = 1;
	requester -> newid = 0;

	requester->next_req_time.tv_sec = 0;
	requester->next_req_time.tv_usec = 0;

}


/* 
   cancel timers here	
   runting: actually i need a ref count mechanism, and unless the ref count is 0, 
   i shouldn't free the data structure...
*/ 
void requester_cleanup(secflow_requester* requester){       
}



/*
  Performs source specific actions upon receiving a packet.  Includes
  updating the current capability state if the packet is a reply. 
  We also increment the requested class if we received a reply.  
*/
void requester_process_hdr(secflow_requester* requester, hdr_secflow *hdr){
        char is_newer;

	is_newer = compare_capability(&requester->latest_cap, &(hdr->cap_reply)) >= 0;
	if(  is_cap_reply(hdr) && is_newer) {
	        
	  //you always record the latest capability

	  memcpy(&requester->latest_cap, &(hdr->cap_reply), sizeof(secflow_capability));
	  setcurrouter(&requester->latest_cap, 0);
	  
	 // printf("requester: got new capability reply, id = %d\n", hdr->cap_reply.id);
	  
	  if(hdr->cap_reply.id == requester->newid){
	    
	    
	    //try next class
	    if(requester -> requesting_class < requester -> app_req_class) 
	      requester->requesting_class ++; 

	    requester -> first_req_4_cur_class = 1;    
	    //otherwise, ... it must have timed out half way in timer_cancel...
	  }
	}
	
}

/*
  Function called to perform the requester specific functionality needed before a 
  packet is sent.  Deals with copying capability from "current" to "latest"
  and taking care of "request" issues.  
  Also sets a timer to timeout in case we do not get a new capability for some reason. 

*/
void requester_initialize_hdr(secflow_requester* requester, hdr_secflow *hdr){
        struct timeval t, now;

	assert(requester != NULL);
	assert(hdr != NULL);
     
	
	//copy just acquired capability to be the "current" capability
	memcpy(&hdr ->cap_current, &requester->latest_cap, sizeof(secflow_capability));
	setcurrouter(&hdr ->cap_current, 0);
	
//	printf("requester: current capability id = %d\n", hdr ->cap_current.id);
	
	//every packet is a request packet, unless application only wants best effort
	if(requester -> app_req_class == 0 || requester->requesting_class == 0) return;
	
	click_gettimeofday(&now);	
//	t = make_timeval(0, SECFLOW_REQ_TIMEOUT * 1000 );

	// Send a request for a new capability
//	if(lessthan(&requester -> next_req_time, &now) || requester -> latest_cap.id == 0){
		
//		timeradd(&now, &t, &requester -> next_req_time);
		
		set_request_bit(hdr);
		setprio(&hdr -> cap_next, requester->requesting_class);
		setcurrouter(&hdr -> cap_next, 0);
		
		if(requester -> latest_cap.id == 0){
		  set_has_token(hdr);
		}
	     
		// If we have a new capability
		if(requester -> first_req_4_cur_class){
			//schedule timeout
			requester -> first_req_4_cur_class = 0;
			requester -> newid ++;
			hdr -> cap_next.id = requester -> newid;
//			timer_reschedule(&requester->timer, SECFLOW_REQUESTER_TIMEOUT);
		}else{
			hdr -> cap_next.id  = requester -> newid;
		}
//		printf("requester: requesting capability id = %d, class = %d\n",
//		       hdr ->cap_next.id, getprio(&hdr -> cap_next) );
//	}
}


/*
  Didn't receive cap_reply and timed out..
  This timer is set in requester_send_packet and cleared if we 
  receive a reply in time by requester_recv_packet.  

void requester_timeout(unsigned long data){
        secflow_requester* requester = (secflow_requester*) data;
	spin_lock_bh(&requester->requester_lock); 

	// 0 is best effort, you don't really need a capability for best effort
	if( requester->requesting_class > 1 ){ 

		requester -> requesting_class --;
		requester -> first_req_4_cur_class = 1;
	}
	
	spin_unlock_bh(&requester->requester_lock); 
	
}
*/

/*
  Change the requesting class of the flow.  
  DW: I don't believe this function is currently used
*/
void requester_set_req_class(secflow_requester* requester, u8 reqclass){

	assert(requester != NULL);

	if(reqclass != requester -> requesting_class){    
		requester->requesting_class = reqclass;
		requester -> first_req_4_cur_class = 1;
	}
	
	requester -> app_req_class = reqclass;
}

CLICK_ENDDECLS

ELEMENT_PROVIDES(bcap_requester)
