
#include "fake_bigint.hh"
#include <click/config.h>

#ifdef CLICK_LINUXMODULE
#include <linux/string.h>
#else
#include <string.h>
#endif

void mpz_square (MP_INT *res, const MP_INT *arg)
{
  MP_INT tmp, *r;

  if (!arg->_mp_size) {
    res->_mp_size = 0;
    return;
  }
  if (res == arg) {
    r = &tmp;
    mpz_init (r);
  }
  else
    r = res;

  int asize = arg->_mp_size > 0 ? arg->_mp_size : -1 * arg->_mp_size;
  const mp_limb_t *ap = arg->_mp_d;

  int rsize = 2 * asize;
  if (r->_mp_alloc < rsize)
    _mpz_realloc (r, rsize);
  mp_limb_t *rp = r->_mp_d;

  if (asize < 24) {
    /* This seems faster to be faster for small sizes. */
    mpn_mul_n (rp, ap, ap, asize);
  }
  else {
    memset(rp, 0x0, rsize * sizeof (mp_limb_t));
    for (int i = 1; i < asize; i++) {
      mp_limb_t *mrp = rp + (i<<1) - 1;
      mrp[asize - i] = mpn_addmul_1 (mrp, ap + i, asize - i, ap[i - 1]);
    }
    mpn_lshift (rp, rp, rsize, 1);
    for (int i = 0; i < asize; i++) {
      mp_size_t rpos = i << 1;
      mp_limb_t c = mpn_addmul_1 (rp + rpos, &ap[i], 1, ap[i]);
      mpn_add_1 (rp + rpos + 1, rp + rpos + 1, rsize - rpos - 1, c);
    }
  }

  while (rsize && !rp[rsize - 1])
    rsize--;
  r->_mp_size = rsize;
  if (res == arg) {
    mpz_clear (res);
    *(MP_INT *) res = tmp;
  }
}


int jacobi(const bigint &a, const bigint &b) {
	return mpz_jacobi(&a, &b);
}

bigint invert(const bigint &a, const bigint &mod) {
	bigint result, gcd;
	mpz_gcdext(&gcd, &result, NULL, &a, &mod);
	if(gcd._mp_size != 1 || gcd._mp_d[0] != 1) 
		result._mp_size = 0;
	else if (result._mp_size < 0)
		mpz_add(&result, &result, &mod);
	return result;
}

u_long mod(const bigint &a, u_long b) {
	static bigint junk;
	return mpz_mod_ui(&junk, &a, b);
}

void swap(bigint &a, bigint &b) {
	a.swap(&b);
}
CLICK_ENDDECLS
ELEMENT_PROVIDES(fake_bigint)
