#ifndef secflow_commondef_h
#define secflow_commondef_h


#include <asm/types.h>
// add some types that aren't normall exported
// by the kernel
typedef __u8 u8;
typedef __u16 u16;
typedef __u32 u32;
typedef __u64 u64;

typedef __s8 s8;
typedef __s16 s16;
typedef __s32 s32;
typedef __s64 s64;

#define BITS_PER_LONG 32
#include <linux/stddef.h>
#include <click/config.h>
#include <click/error.hh>
#include <click/confparse.hh>
#include <click/glue.hh>

#ifdef CLICK_LINUXMODULE

#define assert(x) do {if(!(x)) click_chatter("secflow assertion failure, at %s line %d\n", __FILE__, __LINE__);} while(0) 

#endif

#define ADDTOKEN_PAINT 99
#define ADD_INVALID_TOKEN_PAINT 100
#define NUM_LEGIT_SENDERS 50

#define EVIL_PORT 6666
#define MAGIC_EXPER_FINISHED_PORT 9999
#define FP_ROUTER_DEBUG	 0 

#define IPPROTO_FANFARE 139
#define TOKEN_REQUEST_PORT 99

//#define CAP_BESTEFFORT 0

#define MAXPATHLEN 8

#define MAXNUMCLASS 2

#define KEY_UPDATE_INTERVAL 1000 // in ms dw: this seems fast

// type of bcap token
#define BTT_SimpleSecret 0x1
#define BTT_PublicKey 0x2
#define BTT_HashChain 0x3
#define BTT_Iterative 0x4


typedef struct t_bcap_token_hdr {
  u16 type;
  u16 size;
} bcap_token_hdr;

typedef struct t_pubkey_token {
  u32 identifier;
  u32 expire_timestamp;
  u32 salt;
} pubkey_token;


typedef struct t_secflow_marking{
	u8 marking; // keyid + MAC, each 4 bits
} secflow_marking;

static inline u8 getkeyid(secflow_marking *m){
	assert(m != NULL);
	return m->marking >> 4;
}

static inline u8 getauthcode(secflow_marking *m){
	assert(m != NULL);
	return m->marking & 0x0f;
}

static inline void setkeyid(secflow_marking *m, u8 keyid){
	assert(m != NULL);
	m->marking = (m->marking & 0x0f) | (keyid << 4);
}

static inline void setauthcode(secflow_marking *m, u8 auth){
	assert(m != NULL);
	m->marking = (m->marking & 0xf0) | auth;
}

// main capability data-structure
typedef struct t_secflow_capability{
	u32 id;
	u16 prioclass;
	u16 cur_router;
	secflow_marking markings[MAXPATHLEN];
}secflow_capability;


static inline int compare_capability(secflow_capability *cap_old, secflow_capability *cap_new){
	assert(cap_old != NULL);
	assert(cap_new != NULL);
	if(cap_new -> id == cap_old -> id) return 0;
	return 1; //always go for cap_new
}

static inline u32 getprio(secflow_capability *c){
	assert(c != NULL);
	return c->prioclass;
}

static inline u8 getcurrouter(secflow_capability *c){
	assert(c != NULL);
	return c->cur_router;
}

static inline void setprio(secflow_capability *c, u32 p){
	assert(c != NULL);
	c->prioclass = p;
}

static inline void setcurrouter(secflow_capability *c, u8 r){
	assert(c != NULL);
	c-> cur_router = r;
}

static inline void inccurrouter(secflow_capability *c){
	assert(c != NULL);
	c->cur_router ++;
}

//Runting: request fail bit is to denote that the current request hasn't been granted by the routers.
//prev request rej bit is for the receiver to send feedback to the sender about the prev request.
//we can use more feedback bits. but right now, it's just 1 bit
//actually... i am not using feedback bits yet... now we reply solely on the sender to time out.
#define HDR_REQUEST_MASK 0x01
#define HDR_CAP_REPLY_MASK 0x02
#define HDR_REQUEST_FAIL_MASK 0x04
#define HDR_PREV_REQUEST_REJ_MASK 0x08
#define HDR_NODATA_MASK 0x10
#define HDR_TCPUDP_MASK 0x20
#define HDR_CHEAT_MALICIOUS_MASK 0x40
#define HDR_HAS_TOKEN 0x80


// This is a full capability header
typedef struct t_hdr_secflow {
	
	u32 controlbits; // secflow_request + secflow_capreply + request_fail + prev_request_rejected + nodata	
	secflow_capability cap_current;
	secflow_capability cap_next;
	secflow_capability cap_reply;
		
} hdr_secflow;



static inline void init_hdr(hdr_secflow* hdr){
	assert(hdr != NULL);
	memset(hdr, 0x0, sizeof(hdr_secflow));
}

static inline int is_request(hdr_secflow* hdr){
	assert(hdr != NULL);
	return hdr -> controlbits & HDR_REQUEST_MASK;
}

static inline void set_request_bit(hdr_secflow* hdr){
	assert(hdr != NULL);
	hdr -> controlbits |= HDR_REQUEST_MASK;
}

static inline int request_failed(hdr_secflow* hdr){
	assert(hdr != NULL);
	return hdr -> controlbits & HDR_REQUEST_FAIL_MASK;
}

static inline void set_request_fail_bit(hdr_secflow* hdr){
	assert(hdr != NULL);
	hdr -> controlbits |= HDR_REQUEST_FAIL_MASK;
}

static inline int is_cap_reply(hdr_secflow* hdr){
	assert(hdr != NULL);
	return hdr -> controlbits & HDR_CAP_REPLY_MASK;
}

static inline void set_cap_reply_bit(hdr_secflow* hdr){
	assert(hdr != NULL);
	hdr -> controlbits |= HDR_CAP_REPLY_MASK;
}

static inline int prev_request_rejected(hdr_secflow* hdr){
	assert(hdr != NULL);
	return hdr -> controlbits & HDR_PREV_REQUEST_REJ_MASK;
}

static inline void set_prev_rej_bit(hdr_secflow* hdr){
	assert(hdr != NULL);
	hdr -> controlbits |= HDR_PREV_REQUEST_REJ_MASK;
}

static inline int has_no_data(hdr_secflow* hdr){
	assert(hdr != NULL);
	return hdr -> controlbits & HDR_NODATA_MASK;
}

static inline void set_no_data(hdr_secflow* hdr){
	assert(hdr != NULL);
	hdr -> controlbits |= HDR_NODATA_MASK;
}

static inline int is_udp(hdr_secflow* hdr){
        assert(hdr != NULL);
        return hdr -> controlbits & HDR_TCPUDP_MASK;
}

static inline void set_udp(hdr_secflow* hdr){
        assert(hdr != NULL);
        hdr -> controlbits |= HDR_TCPUDP_MASK;
}

static inline int is_malicious(hdr_secflow* hdr){
        assert(hdr != NULL);
        return hdr -> controlbits & HDR_CHEAT_MALICIOUS_MASK;
}

static inline void set_malicious(hdr_secflow* hdr){
        assert(hdr != NULL);
        hdr -> controlbits |=  HDR_CHEAT_MALICIOUS_MASK;
}

static inline int has_token(hdr_secflow* hdr) {
  assert(hdr != NULL);
  return hdr -> controlbits & HDR_HAS_TOKEN;
}

static inline void set_has_token(hdr_secflow* hdr) {
  assert(hdr != NULL);
  hdr->controlbits |= HDR_HAS_TOKEN;
}


#endif
