#ifndef FASTPASS_HH
#define FASTPASS_HH
#include <click/element.hh>
#include <click/gaprate.hh>
#include <click/packet.hh>
#include <click/atomic.hh>
#include <clicknet/udp.h>
#include <click/timer.hh>
#include <click/standard/alignmentinfo.hh>
#include <click/glue.hh>
#include <click/hashmap.hh>
#include <click/bighashmap.hh>
#include "cap_requester.h"
#include "cap_granter.h"
#include <click/vector.hh>
#include "token_store.hh"
#include "token_req.hh"

CLICK_DECLS


enum TokenType { TOKEN_PUBKEY, TOKEN_TVA };

struct flow_info {
        flow_info() {
                granter_init(&granter);
                requester_init(&requester);
        }
        TokenType token_type;
        secflow_requester requester;
        secflow_granter granter;
};



class FastPassEndhost : public Element { 


public:

  FastPassEndhost();
  ~FastPassEndhost();

  static const int INCOMING = 1;
  static const int OUTGOING = 0;
 
  static const int FP_ENDHOST_TIMER_MSEC = 2000;
 
  const char *class_name() const		{ return "FastPassEndhost"; }
  const char *processing() const		{ return PUSH; }

  int initialize(ErrorHandler *errh);
  void run_timer();
  void add_handlers();   
  int configure(Vector<String> &, ErrorHandler *);
  void push(int port, Packet *p);  
  TokenStore _token_store; 
 
protected:
  void handleIncoming(Packet *p); 
  void handleOutgoing(Packet *p); 
  int create_conn_lookup_key(unsigned int local_addr,
		unsigned short local_port, unsigned char proto);
  int outgoing_fill_secflow_buf(flow_info* info,
             struct in_addr *dest_addr, char* secflow_buf, int color);	

  bool parseUDPForTokenReply(Packet *p);  
  void updateChecksum(unsigned char* ip_hdr, u_int ip_hdr_size);
  static int token_write_handler (const String &conf, Element *e, void *, ErrorHandler *errh);

  // maps for secflow data structures.  
  // key is 4 byte value B0 B1 B2 B3 where:
  // B0 - last byte of the local IP address
  // B1 - the layer 3 protocol (TCP or UDP)
  // B2 : B3 - the local port number 
  HashMap<int ,  flow_info> flow_map;
  TokenType token_type_;
  Timer _timer;
  Vector<Packet *> _needs_token_queue;
  TokenRequester _token_requester;
  IPAddress _token_server, _token_req_source;
 
  bool _cksum : 1;
#if HAVE_FAST_CHECKSUM && FAST_CHECKSUM_ALIGNED
  bool _aligned : 1;
#endif
  atomic_uint32_t _id;
};

CLICK_ENDDECLS
#endif
