/*
 * originally based on:  
 *
 * siff.{cc,hh} -- element implements the siff marking and verification 
 * Jalaja Kurubarahalli
 *
 * Copyright (c) 2005-2006 Carnegie Mellon University 
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, subject to the conditions
 * listed in the Click LICENSE file. These conditions include: you must
 * preserve this copyright notice, and you cannot mention the copyright
 * holders in advertising related to the Software without their permission.
 * The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
 * notice is a summary of the Click LICENSE file; the license in that file is
 * legally binding.
 */

#include <click/config.h>
#include "fastpass_router.hh"
#include "fastpass_common.hh"
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>


#ifdef CLICK_LINUXMODULE
# include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
CLICK_CXX_PROTECT
# include <linux/sched.h>
CLICK_CXX_UNPROTECT
# include <click/cxxunprotect.h>
#endif
CLICK_DECLS

#include "token_validate.hh"



FastPassRouter::FastPassRouter()
  : Element(1, 1), keytimer(this)
{
  MOD_INC_USE_COUNT;
  _protocol = IPPROTO_FANFARE;
  
}

FastPassRouter::~FastPassRouter()
{
  MOD_DEC_USE_COUNT;
  keytimer.unschedule();
}

int FastPassRouter::initialize(ErrorHandler *){
 
	keytimer.initialize(this);
	keytimer.schedule_after_ms(KEY_UPDATE_INTERVAL);
	return 0;
}


void FastPassRouter::run_timer(){
	marking_handler.run_timer();	
	keytimer.reschedule_after_ms(KEY_UPDATE_INTERVAL);
}

int
FastPassRouter::configure(Vector<String> &conf, ErrorHandler* errh)
{
 
//  String out_hw_addr;  
  int numclass = MAXNUMCLASS + 1; // 0 = best effort, MAXNUMCLASS = requests
  
  if (cp_va_parse(conf, this, errh,
		  cpOptional,
		  cpInteger, "protocol", &_protocol,
		  cpEnd) < 0)
    return -1;
  
  if(FP_ROUTER_DEBUG)
  	click_chatter("FastPassRouter: Protocol in initialization = %d\n", 
			_protocol);
 
  set_noutputs(numclass);

  return 0;
}

void FastPassRouter::push(int i, Packet *p) {
  	int prioclass;

#ifdef CLICK_LINUXMODULE
    	iphdr *ipheader = p->skb()->nh.iph;
    	hdr_secflow *secflowhdr = (hdr_secflow*)
		((unsigned int)ipheader+(unsigned int)ipheader->ihl*4);
	unsigned char proto = ipheader->protocol;
#else
	click_ip *ipheader = (click_ip *) p->ip_header();
	hdr_secflow *secflowhdr = (hdr_secflow *) p->transport_header();
	unsigned char proto = ipheader->ip_p;
#endif
	prioclass = 0;

	if(proto == IPPROTO_FANFARE) {
	  // mark packet!
	  marking_handler.mark_packet(secflowhdr);

	 if(has_token(secflowhdr)) {
	   prioclass = MAXNUMCLASS; // request output
	 }else{
	    prioclass = marking_handler.classify_packet(secflowhdr);	
	
         }
	  if(prioclass < 0 || prioclass >= noutputs() ) {
	    click_chatter("classifier returned wrong class, prioclass = %d\n", 
				prioclass);
	    return;
	  }

	}
	if(FP_ROUTER_DEBUG)
		click_chatter("prio class %d \n", prioclass);
	output(prioclass).push(p);
}



CLICK_ENDDECLS
ELEMENT_REQUIRES(SecflowRouterMarkingHandling)
EXPORT_ELEMENT(FastPassRouter)
