
#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <clicknet/ip.h>
#include <clicknet/tcp.h>
#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS
#include "fp_test_utils.hh"


WritablePacket *
make_request_packet(IPAddress src, IPAddress dst, u_short src_port,
		u_short dst_port, int data_len) {

    unsigned int headroom = 150;  	
    return make_tcp_packet(src,dst,src_port,dst_port, data_len, 
				headroom, TH_SYN);
}

WritablePacket *
make_data_packet(IPAddress src, IPAddress dst, u_short src_port,
		u_short dst_port, int data_len) {

    unsigned int headroom = 60;                                                           
    return make_tcp_packet(src,dst,src_port,dst_port,  data_len, 
				headroom, TH_ACK);
}

WritablePacket *
make_tcp_packet(IPAddress src, IPAddress dst, u_short src_port,
			u_short dst_port, unsigned int data_len,
			unsigned int headroom, unsigned short flags) {
    unsigned char buf[1500];
    memset(buf, 0xD, 1500);	              

    int hdr_size = sizeof(click_ip) + sizeof(click_tcp);                                                                                                 
    WritablePacket *tcp_packet = Packet::make(headroom, buf, data_len, 0);
    tcp_packet = tcp_packet->push(hdr_size);
    click_ip *ip =
      reinterpret_cast<click_ip *>(tcp_packet->data());
    click_tcp *tcp = reinterpret_cast<click_tcp *>(ip + 1);
    // set up IP header
    ip->ip_v = 4;
    ip->ip_hl = sizeof(click_ip) >> 2;
    ip->ip_len = htons(sizeof(click_tcp) + data_len);
    ip->ip_id = 0;
    ip->ip_p = IP_PROTO_TCP;
    ip->ip_src = src.in_addr();
    ip->ip_dst = dst.in_addr();
    ip->ip_tos = 0;
    ip->ip_off = 0;
    ip->ip_ttl = 250;
    ip->ip_sum = 0;
    ip->ip_sum = 0; // will be recalculated in lower fastpass layer
    tcp_packet->set_dst_ip_anno(dst);
    tcp_packet->set_ip_header(ip, sizeof(click_ip));
                                                                                                                       
    // set up TCP header
    tcp->th_sport = htons(src_port);
    tcp->th_dport = htons(dst_port);
    tcp->th_seq = random();
    tcp->th_ack = random();
    tcp->th_off = sizeof(click_tcp) >> 2;
    tcp->th_flags = flags;
    tcp->th_win = 65535;
    tcp->th_urp = 0;
    tcp->th_sum = 0; // this packet will never be touched by a real tcp stack
    return tcp_packet;                                                                                               
}


CLICK_ENDDECLS
ELEMENT_PROVIDES(fp_test_utils)

