#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <clicknet/ip.h>
#include <clicknet/tcp.h>
#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS

#include <click/packet_anno.hh>
#include "fp_thruput_test_sink.hh"
#include "fp_test_utils.hh"

FPThroughputTestSink::FPThroughputTestSink() : Element(1,0), _timer(this)
{
	MOD_INC_USE_COUNT;
}

FPThroughputTestSink::~FPThroughputTestSink() 
{
	MOD_DEC_USE_COUNT;
}

int FPThroughputTestSink::configure(Vector<String> &conf,
	ErrorHandler *errh) 
{
	_num_req = 0;
	_num_data = 0;
	return 0;
}
  
int FPThroughputTestSink::initialize(ErrorHandler *e) 
{


	timer_interval = 1000; 
  	_timer.initialize(this);
  	_timer.schedule_after_ms(timer_interval);
	_last_data_len = 0;
	return 0;
}


void 
FPThroughputTestSink::push(int i, Packet *p) {

	// find out whether it is a data or request packet

	click_ip *ip = (click_ip*) (p->data());
	if(ip->ip_p != IPPROTO_TCP) {
		click_chatter("got odd proto: %d \n", ip->ip_p);
		p->kill();
		return;
	}


	click_tcp *tcp = (click_tcp*)(p->data() + sizeof(click_ip));
	u_short dst_port = ntohs(tcp->th_dport);
	if(dst_port == 30) _num_req++;
	else if(dst_port == 40) {
		_last_data_len = ntohs(ip->ip_len);	
		_num_data++;
	} else {
		click_chatter("received odd dst port of %d \n", dst_port);
	} 
	p->kill();
}


void
FPThroughputTestSink::run_timer()
{

   click_chatter("sink: reqs = %d full = %d (data len = %d)\n", 
		_num_req, _num_data, _last_data_len); 
   _num_req = 0;
   _num_data = 0;
  _timer.schedule_after_ms(timer_interval);
}

CLICK_ENDDECLS
EXPORT_ELEMENT(FPThroughputTestSink)
