#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <clicknet/ip.h>
#include <clicknet/tcp.h>
#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS

#include <click/packet_anno.hh>
#include "fp_thruput_test_source.hh"
#include "fp_test_utils.hh"
#include "fastpass_common.hh"

FPThroughputTestSource::FPThroughputTestSource() : Element(0,1), _timer(this),
 _req_rate(), _data_rate()
{
	MOD_INC_USE_COUNT;
}

FPThroughputTestSource::~FPThroughputTestSource() 
{
	MOD_DEC_USE_COUNT;
}

int FPThroughputTestSource::configure(Vector<String> &conf,
	ErrorHandler *errh) 
{

	if(cp_va_parse(conf, this, errh,
                  cpIPAddress, "source addr", &_src,
		  cpIPAddress, "dest addr" , &_dst,
                  cpEnd) < 0)
		return -1;

	_request_packet = make_request_packet(_src, _dst,
				30, 30, 20);		

	SET_PAINT_ANNO(_request_packet, ADDTOKEN_PAINT);

	_data_packet = make_data_packet(_src, _dst,
				40, 40 , 1200);
	SET_PAINT_ANNO(_data_packet, 0);

	_req_rate.set_rate(1,errh);
        _data_rate.set_rate(1,errh);
	return 0;
}
  
int FPThroughputTestSource::initialize(ErrorHandler *e) 
{

	timer_interval = 5; 
  	_timer.initialize(this);
  	_timer.schedule_after_ms(timer_interval);

	return 0;
}


void
FPThroughputTestSource::run_timer()
{
  struct timeval now;
  click_gettimeofday(&now);
  _request_packet->set_timestamp_anno(now); 
  _data_packet->set_timestamp_anno(now);

  while(_req_rate.need_update(now)) {
	output(0).push(_request_packet->clone());
        _req_rate.update();
  }

  while(_data_rate.need_update(now)) {
	output(0).push(_data_packet->clone());
        _data_rate.update();
  }

 
  _timer.schedule_after_ms(timer_interval);
}


void FPThroughputTestSource::add_handlers(){
        add_write_handler("set_request_rate", request_rate_write_handler, 0);
        add_write_handler("set_data_rate", data_rate_write_handler, 0);
	add_write_handler("set_data_size", data_size_write_handler, 0);
}


int FPThroughputTestSource::request_rate_write_handler(
const String &conf, Element *e, void *, ErrorHandler *errh) {
  	FPThroughputTestSource *source = (FPThroughputTestSource*)e;        
        
	unsigned int r_rate;
        const char *s = conf.c_str();
        cp_unsigned(s,s + conf.length(), 10 , &r_rate);
        if(r_rate >= 0) {
        	source->_req_rate.set_rate(r_rate);
	        return 0;
        }

        return 1;
}


int FPThroughputTestSource::data_rate_write_handler(
const String &conf, Element *e, void *, ErrorHandler *errh) {
  	FPThroughputTestSource *source = (FPThroughputTestSource*)e;        
	unsigned int d_rate;
        const char *s = conf.c_str();
        cp_unsigned(s,s + conf.length(), 10 , &d_rate);
        if(d_rate >= 0) {
        	source->_data_rate.set_rate(d_rate);
	        return 0;
        }

        return 1;
}

int FPThroughputTestSource::data_size_write_handler(
const String &conf, Element *e, void *, ErrorHandler *errh) {
  	FPThroughputTestSource *source = (FPThroughputTestSource*)e;        
	unsigned int d_size;
        const char *s = conf.c_str();
        cp_unsigned(s,s + conf.length(), 10 , &d_size);
        if(d_size > 60 ) {
		// should probably lock
		Packet *old = source->_data_packet;
		source->_data_packet = make_data_packet(
				source->_src, source->_dst,
				40, 40 , d_size);
		SET_PAINT_ANNO(source->_data_packet, 0);
		old->kill();
	        return 0;
        }

        return 1;
}
CLICK_ENDDECLS
EXPORT_ELEMENT(FPThroughputTestSource)
