#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <clicknet/ip.h>
#include <clicknet/tcp.h>
#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS

#include <click/packet_anno.hh>
#include "fp_ttc_test_sink.hh"
#include "fp_test_utils.hh"
#include "fastpass_common.hh"

FPTTCTestSink::FPTTCTestSink() : Element(1,1), _timer(this)
{
	MOD_INC_USE_COUNT;
}

FPTTCTestSink::~FPTTCTestSink() 
{
	MOD_DEC_USE_COUNT;
}

int FPTTCTestSink::configure(Vector<String> &conf,
	ErrorHandler *errh) 
{
	return 0;
}
  
int FPTTCTestSink::initialize(ErrorHandler *e) 
{

	_num_attack = _num_legit = 0;
	timer_interval = 1000; 
  	_timer.initialize(this);
  	_timer.schedule_after_ms(timer_interval);
	return 0;
}


void 
FPTTCTestSink::push(int i, Packet *p) {

	// find out whether it is a data or request packet

	click_ip *ip = (click_ip*) (p->data());
	if(ip->ip_p != IPPROTO_TCP) {
		click_chatter("got odd proto: %d \n", ip->ip_p);
		p->kill();
		return;
	}

	click_tcp *tcp = (click_tcp*)(p->data() + sizeof(click_ip));
	u_short src_port = ntohs(tcp->th_sport);
	
	if(src_port >= 0 && src_port <= NUM_LEGIT_SENDERS) {
		IPAddress dst = IPAddress(ip->ip_src);
		IPAddress src = IPAddress(ip->ip_dst);
		
		Packet *reply = make_request_packet(src, dst, src_port, src_port, 
			60);
		
		output(0).push(reply);

		_num_legit++;
	}else if(src_port == EVIL_PORT) {
		_num_attack++;
	}else {
		click_chatter("received odd src port of %d \n", src_port);
	} 
	p->kill();
}


void
FPTTCTestSink::run_timer()
{

   click_chatter("sink: legit reqs = %d attack reqs = %d \n", 
		_num_legit, _num_attack); 
   _num_legit = 0;
   _num_attack = 0;
  _timer.schedule_after_ms(timer_interval);

   // TODO: test if we're done, if so, send out magic packet to all hosts!

}

CLICK_ENDDECLS
EXPORT_ELEMENT(FPTTCTestSink)
