#ifndef _fp_ttc_test_source_hh
#define _fp_ttc_test_source_hh

#include <click/element.hh>
#include <click/gaprate.hh>
#include <click/packet.hh>
#include <click/atomic.hh>
#include <clicknet/udp.h>
#include <click/timer.hh>
#include <click/standard/alignmentinfo.hh>
#include <click/glue.hh>
#include <click/hashmap.hh>
#include <click/bighashmap.hh>
#include <click/vector.hh>
#include "fastpass_endhost.hh" // for TokenType


#define SEND_PERIOD_LEN 30   // all legit requests will be in a 30-second range
#define NUM_SUBNETWORKS 20
#define TIMER_INTERVAL_MSEC 10
#define ATTACK_PPS 5


struct ttc_test_flow {
	timeval start;
	timeval ttc;
	timeval timeout;
	Packet *request_packet;
	int id; // this is source port number
	bool reply_received;
	int subnetwork;
};

class FPTTCTestSource : public Element {

public:

	FPTTCTestSource();
	~FPTTCTestSource();

  	IPAddress _src;
  	IPAddress _dst;
	int initialize(ErrorHandler *);
	void push(int port, Packet *p);
	void run_timer();
	const char *class_name() const { return "FPTTCTestSource"; }
	const char *processing() const { return PUSH; }
	int configure(Vector<String> &, ErrorHandler *);
	bool can_live_reconfigure() const { return true; }
        void add_handlers();        

	void init_flows();
	void experiment_done();
  	Timer _timer;
	unsigned int timer_interval; 
	
	int exper_num, total_expers;
	bool exper_active;
	timeval exper_starttime;
	int timer_count;
	int req_sent, req_received; 	
	// used for good requests
	ttc_test_flow flows[NUM_LEGIT_SENDERS];

	// used for attacks
	Packet *attack_packet;
	Vector<int> attack_info;
	int attacker_subnetworks[NUM_SUBNETWORKS];

	static String exper_read_handler(Element *, void*);
	static int exper_write_handler(const String&, Element*, void*,
   ErrorHandler*);

	TokenType token_type;
}; 

#endif
