#ifndef _pubkey_token_handler_hh
#define _pubkey_token_handler_hh

#include <click/timer.hh>
#include <click/element.hh>
#include "token_validate.hh"
#include "fastpass_common.hh"
#include "BloomFilter.h"

#define TIMER_MSEC	1000
#define BLOOM_FILTER_RESET	5
#define FILTER_SIZE_BYTES 500000
class PubkeyTokenHandler : public Element {

public:
	PubkeyTokenHandler();
	~PubkeyTokenHandler();

	static const int MAX_NUM_PUBKEYS = 128;
	int num_pub_keys;
	pubkey_info pub_keys[MAX_NUM_PUBKEYS];
	void run_timer();
	int initialize(ErrorHandler *);
	void pubkey_token_test(int num_verifies);

	const char *class_name() const { return "PubkeyTokenHandler"; }
	const char *processing() const { return PUSH; }
	int configure(Vector<String> &, ErrorHandler *);
	void add_handlers();
	void push(int i, Packet *p);
private:
	static int test_token_write_handler
             (const String &conf, Element *e, void *, ErrorHandler *errh);
        int add_public_key(char* pubkey_str);
        static int pubkey_write_handler
             (const String &conf, Element *e, void *, ErrorHandler *errh);

	Timer _timer;
	bloom_filter _duplicate_filter;
	unsigned int _num_valid, _num_invalid;
	int _filter_reset_counter;
}; 

#endif
