/* $Id: rabin.C,v 1.11 2001/08/19 00:27:16 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

/* Rabin-Williams public key algorithm.  From:
 *    A Modification of the RSA Public-Key Encryption Procedure.
 *    H. C. Williams, IEEE Transactions on Information Theory,
 *    Vol. IT-26, No. 6, November, 1980.
 *
 * The cipher is based on Rabin's original signature scheme.  This is
 * NOT RSA, and is NOT covered by the RSA patent.  (That's not to say
 * they couldn't try to sue you anyway for using this, but it's
 * unlikely.  However, if you use this code, you must agree to assume
 * all responsibility for patent infringement.)
 *
 * Two prime numbers p, q, chosen such that:
 *    p = 3 mod 8
 *    q = 7 mod 8
 *
 * [Note:  This means pq = 5 mod 8, so J(2, pq) = -1.]
 *
 * Define N and k as:
 *    N = pq
 *    k = 1/2 (1/4 * (p-1) * (q-1) + 1);
 *    [k = ((p-1) * (q-1) + 4) / 8;]
 *
 * The public key is (N).
 * The private key is (N, k).
 *
 * E2(M) =  M^2 % N
 *
 *
 *          { (M/4-1)/2      when M = 0 mod 4
 * D1(M) =  { ((N-M)/4-1)/2  when M = 1 mod 4
 *          { (M/2-1)/2      when M = 2 mod 4
 *          { ((N-M)/2-1)/2  when M = 3 mod 4
 *
 * Public key operations on messages M < (N-4)/8:
 *   Verify:   D1(E2(M))
 */

#include "rabin.hh"
#include <click/config.h>

//INITFN (scrubinit);

//static void
//scrubinit ()
//{
//  mp_setscrub ();
//}


void
rabin_pub::D1 (bigint &m, const bigint &in) const
{
  bigint tmp = bigint();
  u_int x = in.getui() & 3; 
 // click_chatter("X = %d \n", x);
  switch (x) {
  case 0:
    mpz_set_si(&tmp,4);
    mpz_sub(&m, &in , &tmp);
    m >>= 3;
    break;
  case 1:
    mpz_sub(&m, &n, &in);
    m -= 4;
    m >>= 3;
    break;
  case 2:
    mpz_set_si(&tmp,2);
    mpz_sub(&m, &in, &tmp);
   // m = in - 2;
    m >>= 2;
    break;
  case 3:
  //  m = n - in;
    mpz_sub(&m, &n, &in);
    m -= 2;
    m >>= 2;
    break;
  }
}

void
rabin_pub::E2 (bigint &m, const bigint &in) const
{
  //m = in * in;
  mpz_square (&m, &in);
  m %= n;
}

CLICK_ENDDECLS
ELEMENT_PROVIDES(rabin)
