// -*-c++-*-
/* $Id: rabin.h,v 1.20 2005/06/17 00:35:45 fubob Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _RABIN_H_
#define _RABIN_H_ 


#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <click/ipaddress.hh>

#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
#include <linux/inet.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS
#include "fake_bigint.hh"
 

class rabin_pub {
public:
  const bigint n;		/* Modulus */

  const int nbits;

  void E2 (bigint &, const bigint &) const;
  void D1 (bigint &, const bigint &) const;

  rabin_pub (const bigint &nn)
    : n (nn), nbits (max ((int) n.nbits () - 5, 0)) {}
  const bigint &modulus () const { return n; }

  // fastpass version of the verify, which assumes that
  // the message is smaller than the key length. 
  bool fastpass_verify(char* msg, int msg_len, const bigint &s) {

	
  //	char buf[128];
  //	mpz_get_str(buf, 16, &n);
  //	click_chatter("in verify, pubkey = %s \n", buf);
	
	bigint r;
        E2(r, s);
//	mpz_get_str(buf, 16, &r);
//	click_chatter("r-between as int = %s \n", buf);
        D1(r, r);
        
	bigint m;
        mpz_set_raw(&m, msg, msg_len);

//	mpz_get_str(buf, 16, &m);
//	click_chatter("m as int = %s \n", buf);
//	mpz_get_str(buf, 16, &r);
//	click_chatter("r as an int = %s \n", buf);

        return (mpz_cmp(&m, &r) == 0);
  }

};

#endif /* !_RABIN_H_  */
