
#include <click/config.h>
#include <click/glue.hh>
#include "secflow_router_marking.hh"
#include "fastpass_common.hh"


SecflowRouterMarking::SecflowRouterMarking(){

	uint32_t i;
	for(i = 0; i < WINDOWSIZE; i++){
		keys[i] = random();	
	}
	key_index = 0;
}


SecflowRouterMarking::~SecflowRouterMarking(){
}

// this is called by the secflow object, which keeps the timer
void SecflowRouterMarking::run_timer(){
  //we may have race conditions here...	
	keys[(key_index + 1) % WINDOWSIZE] = random();
	key_index = (key_index + 1) % WINDOWSIZE;
}


int SecflowRouterMarking::classify_packet(hdr_secflow *hdr){

	char *str = (char *)hdr;	
	int ret = 0;
	
	secflow_capability *cap = &hdr->cap_current;
	u8 index = getcurrouter(cap);
	
	u32 cap_keyid = getkeyid(&cap->markings[index]);
	u8 cap_authcode = getauthcode(&cap->markings[index]);
	inccurrouter(cap);
	
	if(cap->id == 0){//no capability in packet...
	  return 0; // best-effort traffic 		
	}else{
	  // verify the capability

	  
	  if(cap_authcode == (keys[cap_keyid] & 0xf)){
	    // verified capibility, use provided index value
	    ret = getprio(cap);
	    int time_elapsed = (key_index + WINDOWSIZE - cap_keyid) % WINDOWSIZE;
	    
	    //2 is a magic number... downgrade capability over time
	    //dw: do we want to do this?  
	    if( 2 - time_elapsed < 0) ret = ret + 2 - time_elapsed;
	    
	    if(ret < 0) ret = 0;
	    if(ret >= MAXNUMCLASS) ret = MAXNUMCLASS - 1;
//	    click_chatter("verified packet: keyid = %d, authcode = %d\n", cap_keyid, cap_authcode);	    
	  }else if (FP_ROUTER_DEBUG) {
	    click_chatter("wrong auth code: keyid = %d, authcode = %d\n", 
				cap_keyid, cap_authcode);
	  }
	  
	}
	
	return ret;
}

void SecflowRouterMarking::mark_packet(hdr_secflow *hdr){
	
	secflow_capability *cap = &hdr->cap_next;	
	u8 index = getcurrouter(cap);
	
  //    	click_chatter("marking packet: keyid = %d, authcode = %d\n", key_index, keys[key_index] & 0xf);
	setkeyid( &cap->markings[index], key_index);
	setauthcode(&cap->markings[index], keys[key_index] & 0xf);
	inccurrouter(cap);
	
	return;
}



CLICK_ENDDECLS
ELEMENT_PROVIDES(SecflowRouterMarkingHandling)







