
#include <click/config.h>
#include <click/glue.hh>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/straccum.hh>
#include <click/ipaddress.hh>

#ifdef CLICK_LINUXMODULE
#include <click/cxxprotect.h>
#include <linux/highmem.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/ip.h>
#include <linux/inet.h>
CLICK_CXX_PROTECT 
#include <linux/sched.h>
CLICK_CXX_UNPROTECT 
#include <click/cxxunprotect.h>
#endif
CLICK_DECLS

#include "fake_bigint.hh"
#include "token_validate.hh"
#include "rabin.hh"

/*
  Validates the token found in the provided buffer.  Does a lookup for the public key 
  of the provided IP in the database, and then verifies the 
  signature given the plaintext values of i,t, and s in the token.  
*/
bool validate_token(pubkey_info* keys, int num_keys, char* token_buf, int token_len, u_int dest_ip) {
  char msg_str[12];
  char sig_str[512];
   
  memcpy(msg_str, token_buf, 12);
  memcpy(sig_str, token_buf + 12, token_len - 12);

//  bcap_print_bytes(token_buf, token_len);
 
  bigint token_int;
  token_int.setraw(sig_str, token_len - 12);

  // debug only
//  char buf[128];
//  mpz_get_str(buf, 16, &token_int);
//  click_chatter("token sig = %s \n", buf);

  for(int i = 0; i < num_keys; i++) {
    if(keys[i].dest_ip == dest_ip){
      bool is_verified = keys[i].key->fastpass_verify(msg_str, 12, token_int);

//      if(is_verified) click_chatter("Token Verification SUCCESS!\n");
//      else click_chatter("FAILED to Validate Token!\n"); 
      
      return is_verified;
    }
  }
  
  IPAddress error_addr = IPAddress(dest_ip);
  click_chatter("ERROR: destination public key not found for dest = %s \n",  
	 error_addr.unparse().c_str());
  return false;
}




void bcap_print_bytes(char* start, int length) {
	int i = 0;
	click_chatter("trying to print %d bytes \n", length);
	for (; i < length; i++) {
		click_chatter("%d : %d \n", i , *(start + i));
	}
}
CLICK_ENDDECLS
ELEMENT_PROVIDES(token_validate)
