#! /usr/bin/perl -w

# Generate fastpass router with  Fastpass Pubkey-Token, TVA, or SIFF
# configurations.  Configurations are passed in by modifying the 
# "master.txt" file in the local directory.  See that file for the 
# config information.  


if($#ARGV != 0) {
	print "must provide node name as the only parameter \n";
	exit(1);
}

my $node_name = $ARGV[0];

# a few hardcoded values
my $num_subnetworks = 20; 
my $victim_ip = "10.1.3.2";
my $token_server_ip = "10.1.4.7";

# read most params from the master file
open(MASTER, "master.txt");

my $is_valid = 0;

while($line = <MASTER>) {
	$line = substr($line, 0, length($line) - 1);
	if($line eq $node_name) {
		$is_valid = 1;
		last; # break out
	}
}

if(!is_valid) {
	print "Master file does not seem to be valid.  Exiting... \n";
	exit(1);
}


# why doesn't chomp work?
$line = <MASTER>;
my $ip_list_str = substr($line, 0, length($line) - 1);
$line = <MASTER>;
my $config_type = substr($line, 0, length($line) - 1);
$line = <MASTER>;
my $default_gw = substr($line, 0, length($line) - 1);
$line = <MASTER>;
my $routes_str = substr($line, 0, length($line) - 1);
$line = <MASTER>;
my $iface_rate_str = substr($line, 0, length($line) - 1);
my @iface_rates = split(/,/, $iface_rate_str);
$line = <MASTER>;
my $req_rate_str = substr($line, 0, length($line) - 1);
my @req_rates = split(/,/, $req_rate_str);

# these sizes are for a 25 ms RTT, ie, the Barbell topo
my %pk_buf_sizes = ('200Mbps' => '2000',  '100Mbps' => '1000',
			'1000Mbps' => '10000');
my %tva_buf_sizes  = ('200Mbps' => '2000',  '100Mbps'  => '1000',
			'1000Mbps' => '10000');

# list IP address, in order addr0, addr1, etc..
# addresses start with the default route to the root
my @ip_addrs = split(/,/, $ip_list_str);
my @ifaces = ();

my $num_ifaces = @ip_addrs;

for($i = 0; $i < $num_ifaces; $i++) {
   $temp = `/users/danwent/startup/get_iface_by_ip $ip_addrs[$i]`;
   $ifaces[$i] = substr($temp, 0, length($temp) - 1);
}

my @routes = split(/,/, $routes_str);

# setup routes to local ips.
for($i = 0; $i < $num_ifaces; $i++) {
	push( @routes, "$ip_addrs[$i]/32 $num_ifaces");
}

push( @routes, "0.0.0.0/0 $default_gw 0");  ## route to root
my $akamai_iface_num = $num_ifaces - 1;
push( @routes, "10.1.4.0/24 $akamai_iface_num"); ## route to akamai

print "AddressInfo(";
for($i = 0; $i < $num_ifaces; $i++) {
   my $ip = $ip_addrs[$i];
   my @octets = split(/\./, $ip);
   if($i != 0) {
    	print ",";
   }
   my $mac = join(":", ("00","00","00","00","00", $octets[2] . $octets[3]));
   print "\n\t addr$i $ip $mac ";
}

print ");\n";

print "

elementclass InputPath {
 \$iface, \$iface_num, \$addr |

 FromDevice(\$iface) -> Paint(\$iface_num) -> 
 c0 :: Classifier(12/0806 20/0001,
                  12/0806 20/0002,
                  12/0800,
                  -);

 ar :: ARPResponder(\$addr);
 c0[2] -> Strip(14) -> [0]output;     
 c0[0] -> ar -> [1]output; 
 c0[1] -> [2]output;
 c0[3] ->  Discard; // unknown ethtype
};

elementclass IPMain {
 
 //dw: ignore IP broadcasts for now ....
 rt :: LinearIPLookup(\n\t";

print join(",\n\t", @routes);

print "	
      );

 input -> CheckIPHeader() -> rt;     
";

for($i = 0; $i <= $num_ifaces; $i++) {
 print "rt[$i] -> [$i]output;\n";
}

print "
};


elementclass IPOutput {
 \$addr |

      input  -> DropBroadcasts
      -> gio :: IPGWOptions(\$addr)
      -> FixIPSrc(\$addr)
      -> dt :: DecIPTTL
      -> fr :: IPFragmenter(1480)
      -> [0]output; 
      dt[1] -> ICMPError(\$addr , timeexceeded) -> [1]output;
      gio[1] -> ICMPError(\$addr , parameterproblem) -> [1]output;
      fr[1] -> ICMPError(\$addr , unreachable, needfrag) -> [1]output;
};

elementclass PriorityAndShapedOutput {	
	\$iface, \$output_rate, \$req_rate |
	
	scheduler :: PrioSched; 
	input[0] -> BandwidthShaper(\$req_rate) -> [0] scheduler;
	input[1] -> [1] scheduler;
	input[2] ->  [2] scheduler;
	scheduler -> BandwidthShaper(\$output_rate)
	-> ToDevice(\$iface); 
}


";
if($config_type =~ /fastpass/) {
print "
elementclass FastPassOutput {
	\$iface, \$output_rate, \$req_rate, \$buf_size |

	prio_out :: PriorityAndShapedOutput(\$iface, \$output_rate, \$req_rate)
";

	print "
	prio_req_queue :: " . get_queue_name("prio_req", "\$buf_size") . "\n";

	print "-> [0]prio_out;

	prio_data_queue :: " . get_queue_name("prio_data", "\$buf_size") . "\n";

	print "-> [1]prio_out;
	best_effort_queue ::" . get_queue_name("best", "\$buf_size") . "\n";

	print "-> [2]prio_out;
	input -> fp_router:: FastPassRouter(139);
	fp_router[2] -> pk :: PubkeyTokenHandler();
	pk[0] -> best_effort_queue; // failed
	pk[1] -> prio_req_queue; // verified
	
	// note: no policying of capability flow for now   
	fp_router[0] -> best_effort_queue; // best-effort
	fp_router[1] -> prio_data_queue; // valid capability
}
";
} ## end print fastpass

if($config_type =~ /tva/) {
$num_inputs = $num_ifaces + $num_subnetworks;

print "
	elementclass TVAOutput {
	\$iface, \$output_rate, \$req_rate, \$buf_size|
	
	prio_out :: PriorityAndShapedOutput(\$iface, \$output_rate, \$req_rate)
	
        rr :: RoundRobinSched() -> [0]prio_out;
	prio_data_queue :: " . get_queue_name("prio", "\$buf_size") . "\n";

	print "-> [1]prio_out;

	best_effort_queue ::" . get_queue_name("best", "\$buf_size") . "\n";

	print "-> [2]prio_out;

	input -> fp_router :: FastPassRouter(139);
	fp_router[2] -> tva :: TVATokenHandler($num_inputs);
	
	fp_router[0] -> best_effort_queue; // best-effort
	fp_router[1] -> prio_data_queue; ; // valid capability
";

# These are the fair-queue queues used by TVA to handle
# request packets.  There is one per input to the router.  
for($i = 0; $i < $num_subnetworks; $i++) {
	print "tva[$i] -> " . get_queue_name("tva$i", "500") . "\n";
	print " -> [$i]rr \n";
}
for($i = 0; $i < $num_ifaces; $i++) {
	$index = $i + $num_subnetworks;
	print "tva[$index] -> " . get_queue_name("tva$i", "500")  . "\n";
	print "-> [$index]rr \n";
}
$num_inputs = $num_ifaces + $num_subnetworks;
print "
	
}
";
} ## end print tva


## print the main router machinery
## dw: currently, we hack this so these configs work
## for both linuxmodule and userland
print "

//to_host :: ToHost;  // kernel
to_host :: Discard;  // userland 
ipmain :: IPMain();
fp_endhost :: FastPassEndhost($token_server_ip, 
	$ip_addrs[$akamai_iface_num])

";

## add a different traffic source depending on config
if($config_type =~ /thru-source/) {
	print "FPThroughputTestSource($ip_addrs[0], $victim_ip) \n";
} elsif ($config_type =~ /ttc-source/) {
	$ttype = "tva";
	if($config_type =~ /fastpass/) { $ttype = "fastpass"; } 

	print "ttc_source :: FPTTCTestSource($ip_addrs[0],$victim_ip,$ttype) \n";
} elsif ($config_type =~ /tohost/ ) {
	print "FromHost(fake0, 10.0.0.1/8)
	-> from_host_cl :: Classifier(12/0806, 12/0800)
	-> ARPResponder(0/0 1:1:1:1:1:1) 
	-> to_host;
	from_host_cl[1]
	-> Strip(14) -> MarkIPHeader()
	";
} else {
	print  "Idle // no traffic generated by this host\n";
}
 
print "
	-> [0]fp_endhost[0] 
         -> ipmain

//FPTokenReqTest() -> IPPrint(req) -> ipmain

//handle local traffic to us!
ipmain[$num_ifaces] ->  [1]fp_endhost[1] ";

if($config_type =~ /thru-sink/) { 
	print "-> FPThroughputTestSink()\n\n";
} elsif ($config_type =~ /ttc-source/) {
	print " -> ttc_source; // get replies back... \n\n";
} elsif ($config_type =~ /ttc-sink/) {
	print " -> ttc_sink :: FPTTCTestSink() -> [0]fp_endhost; \n";
} elsif ($config_type =~ /tohost/) {
	print "
-> EtherEncap(0x0800, 1:1:1:1:1:1, 01:02:03:04:05:06)
-> to_host \n\n";
} else {
	print "-> Discard; // ignore traffic to localhost\n";
}


  if($config_type =~ /thru-source/) {
	## don't have throughput sources do any token processing
#	$config_type = "bare";
  }

for($i = 0; $i < $num_ifaces; $i++) {
  $input_num = $i + $num_subnetworks;
  print "input_path$i :: InputPath($ifaces[$i], $input_num, addr$i);\n";

   $buf_size = "1000";
  if($config_type =~ /fastpass/) {
    #    $buf_size = $pk_buf_sizes{$iface_rates[$i]};	
        print "fastpass$i :: FastPassOutput($ifaces[$i], $iface_rates[$i], 
		$req_rates[$i], $buf_size)\n";
  }
  elsif($config_type =~ /tva/) {
#	$buf_size = $tva_buf_sizes{$iface_rates[$i]};
        print "fastpass$i :: TVAOutput($ifaces[$i], $iface_rates[$i], 
		$req_rates[$i],	$buf_size)\n";
  }
  elsif($config_type =~ /bare-withratelimit/) {
#	$buf_size = $tva_buf_sizes{$iface_rates[$i]};
	print "fastpass$i :: " . get_queue_name("out$i", "$buf_size") . "\n";
 	print "-> BandwidthShaper($iface_rates[$i]) 
	-> ToDevice($ifaces[$i]); \n";
	
  }
  elsif($config_type =~ /bare/) {
#	$buf_size = $tva_buf_sizes{$iface_rates[$i]};
	print "fastpass$i :: " . get_queue_name("out$i", "$buf_size") . "\n";
	print "  -> ToDevice($ifaces[$i]); \n";
  }
  print "ipmain[$i] -> ipout$i :: IPOutput(addr$i)  \n";
  print " -> arpq$i :: ARPQuerier(addr$i) -> fastpass$i; \n";
  print " ipout$i [1] -> ipmain; // error reporting \n";
  
  print "input_path$i [0] ->  ipmain; // main IP forwarding path \n";
  print "input_path$i [1] -> fastpass$i; // my ARP responses   \n";
  print "input_path$i [2] -> tee$i :: Tee;\n";
  print "tee$i [0] -> to_host; // copy ARP responses to host \n"; 
  print "tee$i [1] -> [1]arpq$i; // also give ARP responses to Click\n\n\n";
}

sub get_queue_name {
  my $name = shift;
  my $size = shift;

#  return "Queue($size)";

  return "BcapNoteQueue($name, $size)";

}

