#!/usr/bin/perl -w

#
# Extremely simple single threaded token server that wraps 
# the 'token_util' binary (which must be in the same directory)
# 
# For any TCP connection, we read the first line and parse it for
# and IP address, and then generate a token that is valid for 5 seconds.
# Conveniently, the parsing also works on HTTP urls that include an 
# IP address as the page name.
#
# Dan Wendlandt 6-2-2006
#

use strict;
use IO::Socket;
use Time::Local;
 
# change this to change how long token is valid
my $token_valid_len_msec = 5000;

if($#ARGV != 2) {
        die "usage [port] [priv.db file] [token_util file] \n";
}

my $key_file = $ARGV[1];
my $exe_file = $ARGV[2];
my ($client,$fp_server);

$fp_server = IO::Socket::INET->new
(
LocalPort => $ARGV[0],
Proto => 'udp',
) or die "Error: couldn't open server. Was port value valid? Are you root?";


while (1) {

	my $request;
	$fp_server->recv($request, 32);	
	my($port,$ipaddr) = sockaddr_in($fp_server->peername);
	my $hishost = gethostbyaddr($ipaddr, AF_INET);
   	print "Client $hishost said ``$request''\n";


	my $rand = int( rand(10000000) );
	my $time = time + $token_valid_len_msec; 
	if($request =~ /\b((\d{1,3}\.){3}\d{1,3})\b/) {
	
		print "requesting token for: '$1' \n";
		my $result = `$exe_file t $rand $time $1 $key_file`;
		print "sending '$result' \n";

		$fp_server->send($result); # perl remembers client info :) 
	} else {
		$fp_server->send("Did not find an IP address in request");
	}
}


