
#define USE_PCTR 0
#include "crypt.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include "token_validate.hh"
#include "token_generate.hh"

#define KEY_SIZE 512 


/*
  Generates a single set of keys.  If you do this, you have to hand
  edit the DB files to insert the new values.  See generate_new_key_DB
*/
void generate_key(char* priv_p_filename, char* priv_q_filename,
		  char* pub_filename) {
  random_update ();
  rabin_priv rabinpriv = rabin_keygen (KEY_SIZE);
  
  str2file(priv_p_filename, rabinpriv.p.getstr(16));
  str2file(priv_q_filename, rabinpriv.q.getstr(16));
  str2file(pub_filename, rabinpriv.n.getstr(16));
}

/*
  Given a list of IP's in a file (one per line), this generates two DB files
  one for private keys and one for public keys. Both have one entry per line. 
  pub key format : <ip>:<n-value> 
  private key format : <ip>:<p-value>:<q-value> 
  Both have all entries separated by comma's for easier parsing within Click
*/
void generate_new_key_DB(char* iplist, char *pubDB, char*privDB) {
  ofstream pub_out(pubDB);
  ofstream priv_out(privDB);
  ifstream in(iplist);
  char buf[24];
  
  while(true) {
    in.getline(buf, 24);
    if(*buf == 0) { break; }  
 
    random_update ();
    rabin_priv rabinpriv = rabin_keygen (KEY_SIZE);
    // pub key format: <ip>:<n-value> 
    pub_out << buf << ":" << rabinpriv.n.getstr(16).cstr() << ",\n";
    
    // private key format: <ip>:<p-value>:<q-value>
    priv_out << buf << ":" << rabinpriv.p.getstr(16).cstr()
	     << ":" << rabinpriv.q.getstr(16).cstr() << "\n";
  }
  
  pub_out.close();
  priv_out.close();
  in.close();

}

/*
  Generates a token file based on the provided values. Token files
  are now text files containing all parameters in ASCII.
  <dest-ip>,<identifier>,<time-stamp>,<salt>,<signature in base-16 hex>
*/
void generate_token(int i, int t, struct in_addr dest_ip,
		    privkey_info* priv_keys, int num_keys) {
  int s = 0; // just temp
  char buf[512];
  char msg_buf[12];
  memcpy((void*)msg_buf, (void*)&i, 4);
  memcpy((void*)(msg_buf+4), (void*)&t, 4);
  memcpy((void*)(msg_buf+8), (void*)&s, 4);
  str msg = str(msg_buf,12);

  for(int j = 0; j < num_keys; j++) {
    if(priv_keys[j].dest_ip == dest_ip) {
      bigint sig = priv_keys[j].key->fastpass_sign(msg);
      str sig_in_hex = sig.getstr(16);
      sprintf(buf, "%s,%d,%d,%d,%s", inet_ntoa(dest_ip), i, t, s,
			sig_in_hex.cstr()); 
      printf("%s\n", buf);
      return;
    }
  }

  printf("Error: could not find the private key for %s \n", 
	 inet_ntoa(dest_ip));
}

/*
  Loads values from a private key database file to the provided array.
  returns the number of entries actually read.  
*/
int load_private_keys(char* pk_file, privkey_info* array, int max) {
  ifstream in(pk_file);
  char buf[512];
  int count = 0;

  while(true) {
    in.getline(buf, 512);
    if(*buf == 0) { break; }
    char *split = strstr(buf, ":");
    char *key_p = split + 1;
    *split = 0; // null terminate
    split = strstr(key_p, ":");
    char *key_q = split + 1;
    *split = 0;


    str key_p_str(strdup(key_p));// leak
    str key_q_str(strdup(key_q));
   
    bigint key_p_int(key_p_str, 16);
    bigint key_q_int(key_q_str, 16);

    array[count].key = new rabin_priv(key_p_int, key_q_int);
    inet_aton(buf, & (array[count].dest_ip));
    count++;
  }
  
  //printf("read in %d private keys \n", count);
  in.close();
  return count;
}


