
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <sys/time.h>
#include "crypt.h"
#include "token_validate.hh"


inline static void timer_sub(timeval *a, timeval *b, timeval *result) {
        result->tv_sec = a->tv_sec - b->tv_sec;
        result->tv_usec = a->tv_usec - b->tv_usec;
        if(result->tv_usec < 0) {
                result->tv_sec--;
                result->tv_usec += 1000000;
        }
}


/*
  Loads public keys from a file and fills in the array of pubkey_info
  values.  Returns the number of entries read.  
*/
int load_public_keys(char* pk_file, pubkey_info* array, int max) {
  ifstream in(pk_file);
  char buf[512];
  int count = 0;

  while(true) {
    in.getline(buf, 512);
    if(*buf == 0) { break; }
    char *split = strstr(buf, ":");
    char *key = split + 1;
    *split = 0; // null terminate
 
    str key_str(strdup(key));
    bigint key_int(key_str, 16);
    array[count].key = new rabin_pub(key_int);
    inet_aton(buf, & (array[count].dest_ip));
    count++;
  }
  
  printf("read in %d public keys \n", count);
  in.close();
  return count;
}

/*
  Simple wrapper around the main validate_token method, which allows a file
  to be specified.  This reads a token as it is stored in a file (perhaps
  for testing), with the destination IP as the first 4 bytes, followed by
  the rest of the token.  
*/
bool validate_token_file(pubkey_info* keys, int num_keys, char* token_filename) {
  ifstream token_in(token_filename);

  char token_buf[128];
  struct in_addr dest_ip;

  // get length of file:
  token_in.seekg (0, ios::end);
  int file_length = token_in.tellg();
  token_in.seekg (0, ios::beg);
  int token_len = file_length - 4;

  token_in.read((char*)&dest_ip, 4);
  token_in.read(token_buf, token_len); 
  
  return validate_token(keys, num_keys, token_buf, token_len, dest_ip);
}

// validates tokens in the new ASCII file-format
bool validate_new_token_file(pubkey_info *keys, int num_keys, 
		char* token_filename) {
        char token_buf[512];

  	ifstream token_in(token_filename);

  	char file_buf[256];

  	// get length of file:
  	token_in.seekg (0, ios::end);
  	int file_length = token_in.tellg();
  	token_in.seekg (0, ios::beg);

  	token_in.read(file_buf, file_length); 
        *(file_buf + file_length) = 0;
	char *cur = file_buf; 
	char *split = strstr(cur, ",");
	*split = 0;
 
        struct in_addr dest;
	inet_aton(cur, &dest);

        unsigned int i, t, s;
	cur = split + 1;
	split = strstr(cur, ",");
	*split = 0;
	i = atoi(cur);

	cur = split + 1;
	split = strstr(cur, ",");
	*split = 0;
	t = atoi(cur);
	
	cur = split + 1;
	split = strstr(cur, ",");
	*split = 0;
	s = atoi(cur);
        printf("loaded i = %d and t = %d s = %d \n", i, t,s);
        
	printf("sig str = %s \n", split + 1);
	bigint sig = bigint(split + 1, 16);

        str sig_str = sig.getraw();

        int token_len = sig_str.len() + 12;
        memcpy(((char*)token_buf) , (void*)&i, 4);
        memcpy(((char*)token_buf) + 4, (void*)&t, 4);
        memcpy(((char*)token_buf) + 8, (void*)&s, 4);
        memcpy(((char*)token_buf) + 12, sig_str.cstr(), sig_str.len());

        return validate_token(keys, num_keys, token_buf, 
		token_len, dest);

	token_in.close();
}


/*
  Validates the token found in the provided buffer.  Does a lookup for the public key 
  of the provided IP in the database, and then verifies the 
  signature given the plaintext values of i,t, and s in the token.  
*/
bool validate_token(pubkey_info* keys, int num_keys, char* token_buf, int token_len,
		    struct in_addr dest_ip) {
 
  str msg_str = str(token_buf, 12);
  str sig_str = str((token_buf + 12), token_len - 12);

  //bcap_print_bytes(token_buf, token_len);
 
  bigint token_int;
  token_int.setraw(sig_str);

  for(int i = 0; i < num_keys; i++) {
    if(keys[i].dest_ip == dest_ip){
     // printf("public key = %s in base 16 \n", 
	//keys[i].key->n.getstr(16).cstr());
      bool is_verified = keys[i].key->fastpass_verify((char*)msg_str.cstr(), 
			msg_str.len(), token_int);

     if(is_verified) printf("Token Verification SUCCESS!\n");
     else printf("FAILED to Validate Token!\n"); 
      
      return is_verified;
    }
  }
  
  printf("ERROR: destination public key not found for dest = %s \n",  
	 inet_ntoa(dest_ip));
  return false;
}



void bcap_print_bytes(char* start, int length) {
	int i = 0;
	for (; i < length; i++) {
		printf("%d : %d \n", i , *(start + i));
	}
}

void rabin_benchmark(int key_size, int num_ops, int numsigs) {
 
  struct timeval start, finish, diff;
  str msg = "hello this is a test message to sign"; 

  // stupid, stupid, stupid... no default constructor for rabin_priv

  bigint p[numsigs];
  bigint q[numsigs];
  bigint s[numsigs];

  for(int i = 0; i < numsigs; i++) {
	random_update();
	rabin_priv temp = rabin_keygen(key_size);
	p[i] = temp.p;
	q[i] = temp.q;
  }


  printf("Done generating %d keys\n", numsigs);

  int num_loops = num_ops / numsigs; // truncate
  int total_ops = numsigs * num_loops; // actual
  // benchmark signing   


  gettimeofday(&start, NULL);
//  for(int j = 0; j < num_loops; j++) {
	for(int i = 0; i < numsigs; i++) {
		rabin_priv priv(p[i],q[i]);
		s[i] = priv.fastpass_sign(msg);
  	}
//  }
  gettimeofday(&finish, NULL);

  timer_sub(&finish, &start, &diff);
  float sign_diff = ((float)diff.tv_sec + (((float)diff.tv_usec)/1000000.0));
  float sign_avg = sign_diff / ((float)total_ops);
  printf("completed %d rabin fastpass-signs in %f, avg = %f , %f per sec \n",
	total_ops, sign_diff, sign_avg, ((float) total_ops)/sign_diff );

  // benchmark verification  
  gettimeofday(&start, NULL);

  for(int i = 0; i < num_loops; i++) {
	for(int j = 0; j < numsigs; j++) {
  		rabin_priv test = rabin_priv(p[j],q[j]);
      		bool b = test.fastpass_verify((char*)msg.cstr(), msg.len(), s[j]);
		if(!b) printf("verification failed \n");
	}
  }

  gettimeofday(&finish, NULL);

  timer_sub(&finish, &start, &diff);
  float verify_diff = ((float)diff.tv_sec + (((float)diff.tv_usec)/1000000.0));
  float verify_avg = verify_diff / ((float)total_ops);
  printf("completed %d rabin fastpass-verifies in %f, avg = %f , %f per sec\n",
	total_ops, verify_diff, verify_avg, ((float)total_ops)/verify_diff );

  
}


